#ifndef FBCDatabaseConnection__H
#define FBCDatabaseConnection__H
#include <stdio.h>
/*!
@header FBCDatabaseConnection

 FBCDatabaseConnection is an agent object which controls access to FrontBase database servers.  An FBCDatabaseConnection can have many distinct sessions to a given database server, though only one session can be active at a time.

 FBCDatabaseConnection provides the basic mechanism for executing SQL queries, fetching result sets, committing transactions, etc.  It is the most common function suite used when writing client applications for FrontBase.
 */

/*!
@function fbcdcConnectToDatabaseRM
 @discussion Like fbcdcConnectToDatabase, except that *errorMsg identifies the error message if an error occurs.  The caller should not free the error message string.
 */
FBCDatabaseConnection* fbcdcConnectToDatabaseRM(char* dbName, char* hostName, char* password, char** errorMsg);

/*!
@function fbcdcConnectToDatabase
 @discussion Establishes a connection to a database on the specified host.  This function requires that the FBExec daemon be running on the specified host to provide the port for the database name.
 @param dbName Name of the database to connect to.
 @param hostName Name of the host the database resides on.
 @param password Digested database password (see fbcDigestPassword).
 @result Returns a connected FBCDatabaseConnection or NULL upon error.  If an error occurs, call fbcdcClassErrorMessage to get the error message.
 */
FBCDatabaseConnection* fbcdcConnectToDatabase(char* dbName, char* hostName, char* password);

/*!
@function fbcdcConnectToDatabaseOnThisHostRM
 @discussion Like fbcdcConnectToDatabaseOnThisHost, except that *errorMsg identifies the error message if an error occurs.  The caller should not free the error message string.
 */
FBCDatabaseConnection* fbcdcConnectToDatabaseOnThisHostRM(char* dbName, char* password, char** errorMsg);

/*!
@function fbcdcConnectToDatabaseOnThisHost
 @discussion A convenience for calling fbcdcConnectToDatabase using localhost.
 */
FBCDatabaseConnection* fbcdcConnectToDatabaseOnThisHost(char* dbName, char* password);

/*!
@function fbcdcConnectToDatabaseUsingPortRM
 @discussion Like fbcdcConnectToDatabaseUsingPort, except that *errorMsg identifies the error message if an error occurs.  The caller should not free the error message string.
 */
FBCDatabaseConnection* fbcdcConnectToDatabaseUsingPortRM(char* hostName, unsigned portNumber, char* password, char** errorMsg);

/*!
@function fbcdcConnectToDatabaseUsingPort
 @discussion Establishes a connection to a database on the specified host and port.  Does not require FBExec to be running on the remote host.
 */
FBCDatabaseConnection* fbcdcConnectToDatabaseUsingPort(char* hostName, unsigned portNumber, char* password);

/*!
@function fbcdcClassErrorMessage
 @discussion In the event of a class level failure, such as a connection failure, calling this function will return a descriptive error message (error messages associated with instances of FBCDatabaseConnection should be retrieved using the returned meta data).
 */
const char* fbcdcClassErrorMessage(void);

/*!
@function fbcdcSetDatabasePassword
 @discussion Sets the database password.  Requires that a session has been established using as the _system user.  The new password must be digested (see fbcDigestPassword).
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcSetDatabasePassword(FBCDatabaseConnection* self, char* newPassword);

/*!
@function fbcdcConnected
 @discussion Indicates whether the given database connection is connected to a database.
*/
Bool fbcdcConnected(FBCDatabaseConnection* self);

/*!
@function fbcdcDatabaseName
 @discussion Returns the database name associated with the given connection.  The caller should not free the returned string.
*/
const char* fbcdcDatabaseName(FBCDatabaseConnection* self);

/*!
@function fbcdcHostName
 @discussion Returns the host name associated with the given connection.  The caller should not free the returned string.
 */
const char* fbcdcHostName(FBCDatabaseConnection* self);

/*!
@function fbcdcStopDatabase
 @discussion Stops the database associated with the given connection.  Requires that a session has been established as the _system user.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
*/
FBCMetaData* fbcdcStopDatabase(FBCDatabaseConnection* self);

/*!
@function fbcdcClose
 @discussion Closes the socket for the given database connection.
 */
void fbcdcClose(FBCDatabaseConnection* self);

/*!
@function fbcdcRelease
 @discussion Releases the resources allocated by an FBCDatabaseConnection.  This should be called when you're finished using each connection.
*/
void fbcdcRelease(FBCDatabaseConnection* self);

/*!
@function fbcdcCreateSession
 @discussion Establishes a new session within a database connection.
 @param self The associated FBCDatabaseConnection.
 @param sessionId The session ID string for the new session.  Typically indicates the name and/or PID of the application establishing the session.
 @param sessionUser The username for the new session, such as "_system".
 @param password The digested user password (see fbcDigestPassword).
 @param systemUser An optional field for the local username establishing the new session (typically, getlogin() or similar).
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcCreateSession(FBCDatabaseConnection* self, char* sessionId, char* sessionUser, char* password, char* systemUser);

/*!
@function fbcdcSetSessionUser
 @discussion Convenience for calling fbcdcCreateSession, using the default session ID.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
*/
FBCMetaData* fbcdcSetSessionUser(FBCDatabaseConnection* self, char* sessionUser, char* password, char* systemUser);

/*!
@function fbcdcSetSessionID
 @discussion Sets the default session ID (within the given database connection) used when establishing new sessions.  The default value is "SessionID".
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
void fbcdcSetSessionID(FBCDatabaseConnection* self, char* anID);

/*!
@function fbcdcSessionID
 @discussion Returns the current session ID for the given connection.
 @result The returned string should not be freed.
*/
const char* fbcdcSessionID(FBCDatabaseConnection* self);

/*!
@function fbcdcSetPasswordForUser
 @discussion Changes the password for the given user.  The passwords must be digested (see fbcDigestPassword).  If the current session user is _system, the old password is ignored.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
*/
FBCMetaData* fbcdcSetPasswordForUser(FBCDatabaseConnection* self, char* userName, char* newPassword, char* oldPassword);

/*!
@function fbcdcSetRollbackAfterError
 @discussion Ask Geert.
 */
Bool fbcdcSetRollbackAfterError(FBCDatabaseConnection* self, Bool flag);

/*!
@function fbcdcSeterrorsFromFBExec
 @discussion Ask Geert.
 */
void fbcdcSeterrorsFromFBExec(FBCDatabaseConnection* self, Bool flag);

/*!
@function fbcdcGeterrorsFromFBExec
 @discussion Ask Geert.
*/
Bool fbcdcGeterrorsFromFBExec(FBCDatabaseConnection* self);

/*!
@function fbcdcDisconnect
 @discussion Disconnect the given session from the given database connection.

 <b>Warning</b>: Ask for clarification regarding these disconnection functions from Geert.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcDisconnect(FBCDatabaseConnection* self, char* sessionID);

/*!
@function fbcdcDisconnectAll
 @discussion Disconnect all sessions from the given database connection.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcDisconnectAll(FBCDatabaseConnection* self);

/*!
@function fbcdcDisconnectDefault
 @discussion Disconnect the default session from the given database connection.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcDisconnectDefault(FBCDatabaseConnection* self);

/*!
@function fbcdcDisconnectCurrent
 @discussion Disconnect the current session from the given database connection.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcDisconnectCurrent(FBCDatabaseConnection* self);

/*!
@function fbcdcSetConnection
 @discussion Ask Geert.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcSetConnection(FBCDatabaseConnection* self, char* sessionID);

/*!
@function fbcdcSetConnection
 @discussion Ask Geert.
 @result The resulting meta data will indicate success or failure and should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcSetConnectionToDefault(FBCDatabaseConnection* self);

/*!
@function fbcdcExecuteDirectSQL
 @discussion Sends the given SQL to the database server for execution.  The SQL statement must end in a semicolon and be NULL terminated.
 @result The resulting meta data will indicate success or failure, as well as the statement type, server execution time, affected row and column count, etc (see FBCMetaData.h).  The meta data object should be released by the caller (see fbcmdRelease).
 */
FBCMetaData* fbcdcExecuteDirectSQL(FBCDatabaseConnection* self, char* sql);

/*!
@function fbcdcExecuteDirectSQLWithLength
 @discussion Similar to fbcdcExecuteDirectSQL but operates on a fixed length SQL buffer.
 */
FBCMetaData* fbcdcExecuteDirectSQLWithLength(FBCDatabaseConnection* self, char* sql, unsigned length);

/*!
@function fbcdcExecuteDirectSQLWithFetch
 @discussion A convenience combining fbcdcExecuteDirectSQL and fbcdcFetch, which saves one roundtrip to the database server.  The rawData pointer will be NULL if an error occurs, or no rows are returned.
 @param self The database connection.
 @param sql The SQL statement to execute.
 @param rowCount The maximum number of rows to return in one batch.  The server can return fewer rows than the maximum if either a memory limitation is reached, or fewer rows were actually identified by the prior select.
 @param rawData A pointer to a pointer for the raw fetch data.  Once fetch processing is complete, *rawData should be freed using free(3).
 */
FBCMetaData* fbcdcExecuteDirectSQLWithFetch(FBCDatabaseConnection* self, char* sql, unsigned rowCount, void **rawData);

/*!
@function fbcdcExecuteDirectSQLWithFetchAndAutoCommit
 @discussion Similar to fbcdcExecuteDirectSQLWithFetch, this is another convenience method intended to reduce roundtrips to the server.  If the entire result set can be returned immediately, this automatically cancels the fetch and performs a commit.  If the result set will require multiple fetch requests to retrieve (which can be determined by comparing the returned rows to the row count in the meta data), the fetch is not automatically cancelled and the caller must fetch, cancel, and commit or rollback as normal.
 */
FBCMetaData* fbcdcExecuteDirectSQLWithFetchAndAutoCommit(FBCDatabaseConnection* self, char* sql, unsigned rowCount, void* *rawData);

/*!
@function fbcdcExecuteDirectSQLWithLengthFetchAndCommit
 @discussion Similar to fbcdcExecuteDirectSQLWithFetchAndAutoCommit, this is another convenience method intended to reduce roundtrips to the server.  If the entire result set can be returned immediately, this automatically cancels the fetch and performs a commit.  If the result set will require multiple fetch requests to retrieve (which can be determined by comparing the returned rows to the row count in the meta data), the fetch is not automatically cancelled and the caller must fetch, cancel, and commit or rollback as normal.
 */
FBCMetaData* fbcdcExecuteDirectSQLWithLengthFetchAndCommit(FBCDatabaseConnection* self, char* sql, unsigned length);

/*!
Obsolete - use fbcdcFetchWithMetadata instead
@function fbcdcFetch
 @discussion Fetches a batch of rows from the server.

 Fetching assumes that some number of rows have been identified in the server using a prior select statement.  The row data is returned in a very compact binary format, which can be extracted using an FBCRowHandler.  The fbcdcFetch function can return fewer rows requested if either a memory limitation is reached, or fewer rows are actually available from the server.  When no more rows are available, fbcdcFetch will return NULL.

 Care should be taken to call fbcdcCancelFetch when fetching is complete, to free up fetch resources on the server side of the connection.

 Also, please note that errors can occur during fetching.  For instance, if the server chooses to evaluate a query lazily (ie., during fetching rather than SQL execution), a SQL function such as a divide could result in a divide by zero.  Because of this, after each call to fbcdcFetch, you must check for an error condition and request the error message if an error is present, otherwise the protocol between the server and client will get out of sync.
 
 @param self The database connection.
 @param rowCount The maximum number of rows to return in a batch.
 @param handleID The fetch handle, retrieved from the meta data of the prior select statement (sent using fbcdcExecuteDirectSQL or similar).

 @result Returns a pointer to the compact fetch result, which should be provided to an FBCRowHandler for row extraction.  Will return NULL both upon error and when no more rows are available, so be sure to check for error conditions upon a NULL return.

     Once associated with an FBCRowHandler, the returned buffer will be freed when the caller releases the row handler, using fbcrhRelease().
 */
void *fbcdcFetch(FBCDatabaseConnection* self, unsigned rowCount, const char* handleID);


/*!
 @function fbcdcFetchWithMetaData
 @discussion Fetches a batch of rows from the server.
 
 Fetching assumes that some number of rows have been identified in the server using a prior select statement.  The row data is returned in a very compact binary format, which can be extracted using an FBCRowHandler.  The fbcdcFetch function can return fewer rows requested if either a memory limitation is reached, or fewer rows are actually available from the server.  When no more rows are available, fbcdcFetch will return NULL.
 
 Care should be taken to call fbcdcCancelFetch when fetching is complete, to free up fetch resources on the server side of the connection.
 
 Also, please note that errors can occur during fetching.  For instance, if the server chooses to evaluate a query lazily (ie., during fetching rather than SQL execution), a SQL function such as a divide could result in a divide by zero.  Because of this, after each call to fbcdcFetch, you must check for an error condition and request the error message if an error is present, otherwise the protocol between the server and client will get out of sync.
 
 @param self The database connection.
 @param rowCount The maximum number of rows to return in a batch.
 @param the metadata for the fetch must be released by the caller, the meta may report errors encountered during the fetch
 @param handleID The fetch handle, retrieved from the meta data of the prior select statement (sent using fbcdcExecuteDirectSQL or similar).
 
 @result Returns a pointer to the compact fetch result, which should be provided to an FBCRowHandler for row extraction.  Will return NULL both upon error and when no more rows are available, so be sure to check for error conditions upon a NULL return.
 
 Once associated with an FBCRowHandler, the returned buffer will be freed when the caller releases the row handler, using fbcrhRelease().
 */
void* fbcdcFetchWithMetaData(FBCDatabaseConnection* self, unsigned rowCount, FBCMetaData** fetchMetaData, const char* handleID);

/*!
@function fbcdcCancelFetch
 @discussion Frees resources associated with the fetch within the server.
 */
FBCMetaData* fbcdcCancelFetch(FBCDatabaseConnection* self, const char* handleID);

/*!
@function fbcdcWriteBLOB
 @discussion Sends a stream of data to the server for storage as a complete BLOB.
 @result The returned FBCBlobHandle can be queried for its textual handle using fbcbhGetHandle.  This handle can be substituted in a SQL string to represented an inserted blob value.  When finished, the caller should call fbcbhRelease to release the FBCBlobHandle.
 */
FBCBlobHandle *fbcdcWriteBLOB(FBCDatabaseConnection* self, void *blobData, unsigned blobDataLength);

/*!
@function fbcdcWriteBLOBFromFile
 @discussion Sends the contents of the named file to the server for storage as a complete BLOB.
 @result The returned FBCBlobHandle can be queried for its textual handle using fbcbhGetHandle.  This handle can be substituted in a SQL string to represented an inserted blob value.  When finished, the caller should call fbcbhRelease to release the FBCBlobHandle.
 */
FBCBlobHandle* fbcdcWriteBLOBFromFile(FBCDatabaseConnection* self, char* fileName, unsigned fileSize);
FBCBlobHandle* fbcdcWriteBLOBFromFILE(FBCDatabaseConnection* self, FILE* f,        unsigned fileSize);

/*!
@function fbcdcReadBLOB
 @discussion Reads a complete BLOB from the server, represented by the given blob handle.
 @result The byte stream representing the BLOB, or NULL upon error.
 */
void *fbcdcReadBLOB(FBCDatabaseConnection* self, FBCBlobHandle *blobHandle);

/*!
@function fbcdcReadBLOBChunk
 @discussion Reads a chunk of a BLOB from the server.
 @param self The database connection.
 @param blobHandle The blob handle representing the BLOB.
 @param offset The offset at which you want to start reading from the BLOB.
 @param chunkSize The size of the chunk you wish to read.  The actual bytes read can be less than the requested amount.  This value will be updated to reflect the actual bytes read upon return.
 @param chunkBuffer The caller provided buffer to hold the BLOB chunk.  Must be at least as large as the given chunkSize.
 @result A byte stream representing a chunk of a BLOB.  NULL is returned upon error, or if no more BLOB data is available.
 */
void *fbcdcReadBLOBChunk(FBCDatabaseConnection* self, FBCBlobHandle *blobHandle, unsigned offset, unsigned *chunkSize, void *chunkBuffer);

/*!
@function fbcdcWriteCLOB
 @discussion Similar to fbcdcWriteBLOB, but intended for CLOB content.
 */
FBCBlobHandle *fbcdcWriteCLOB(FBCDatabaseConnection* self, char* aString);

/*!
@function fbcdcWriteCLOBWithLength
 @discussion Similar to fbcdcWriteCLOB, but operates on a fixed length string rather than a NULL terminated string.
 */
FBCBlobHandle *fbcdcWriteCLOBWithLength(FBCDatabaseConnection* self, char* aString, unsigned clobDataLength);

/*!
@function fbcdcReadCLOB
 @discussion Similar to fbcdcReadBLOB, but intended for CLOB content.
 */
unsigned char* fbcdcReadCLOB(FBCDatabaseConnection* self, FBCBlobHandle *blobHandle);

/*!
@function fbcdcWriteCHARACTERWithLength
 @discussion Similar to fbcdcWriteCHARACTER, but intended for fixed length strings.
 */
FBCBlobHandle *fbcdcWriteCHARACTERWithLength(FBCDatabaseConnection* self, char* aString, int strLen);

/*!
@function fbcdcWriteCHARACTER
 @discussion Similar to fbcdcWriteCLOB, but intended for large character content (ie., char, varchar).
 */
FBCBlobHandle *fbcdcWriteCHARACTER(FBCDatabaseConnection* self, char* aString);

/*!
@function fbcdcErrorMetaData
 @discussion Returns the error meta data for the database connection.  You should call this after encountering an error, and then extract the textual error message using fbcemdAllErrorMessages.
 @result The returned error meta data must be released using fbcemdRelease.
 */
FBCErrorMetaData* fbcdcErrorMetaData(FBCDatabaseConnection* self,const FBCMetaData* meta);

/*!
@function fbcdcErrorMessage
 @discussion Returns the class level error message associated with the database connection.  You should call this when you encounter an error with a class level function, such as fbcdcConnectToDatabase.
 @result The returned error string should not be freed by the caller.
 */
const char* fbcdcErrorMessage(FBCDatabaseConnection* self);

/*!
@function fbcdcCommit
 @discussion A shortcut for calling fbcdcExecuteDirectSQL(fbdc, "commit;").
 */
FBCMetaData* fbcdcCommit(FBCDatabaseConnection* self);

/*!
@function fbcdcRollback
 @discussion A shortcut for calling fbcdcExecuteDirectSQL(fbdc, "rollback;").
 */
FBCMetaData* fbcdcRollback(FBCDatabaseConnection* self);

/*!
@function fbcdcStopAgent
 @discussion Kills and disconnects another agent in the database server (similar to killing a process under a Unix operating system).  Requires an established session as the _system user.
 */
FBCMetaData* fbcdcStopAgent(FBCDatabaseConnection*self, const char* agentID);

/*!
@function fbcdcStopAgentsBusyCondition
 @discussion Requests that the given agent stop its current operation within the database server (such as executing a long query).  This will not interrupt certain conditions such as waiting for a lock.
 */
FBCMetaData* fbcdcStopAgentsBusyCondition(FBCDatabaseConnection*self, const char* agentID);

/*!
@function fbcdcGetDatabaseLogTail
 @discussion Returns the most recent entry in the database log file.
 @result The log entry is stored in the message portion of the returned FBCMetaData, which can be retrieved by using fbcmdMessage.  As usual, the returned FBCMetaData must be released when done.
 */
FBCMetaData* fbcdcGetDatabaseLogTail(FBCDatabaseConnection*self);

/*!
@function fbcdEnableInterrupts
 @discussion Enables interrupts for the connection.
 */
FBCMetaData* fbcdEnableInterrupts(FBCDatabaseConnection* self);

/*!
@function fbcdcInterruptAgent
 @discussion Interrupts activity on current connection.
 */
void fbcdcInterruptAgent(FBCDatabaseConnection* self);

#define FBC_Undefined	-1
#define FBC_ISO8859_1	 1

int  fbcdcInputCharacterSet(FBCDatabaseConnection* self);

void fbcdcSetInputCharacterSet(FBCDatabaseConnection* self, int charSet);

int  fbcdcOutputCharacterSet(FBCDatabaseConnection* self);

void fbcdcSetOutputCharacterSet(FBCDatabaseConnection* self, int charSet);

Bool         fbcdcWriteDataDirect(FBCDatabaseConnection* self, void* data, unsigned length);
FBCMetaData* fbcdcMetaDataDirect(FBCDatabaseConnection* self);

FBCMetaData* fbcdcMemberNames(FBCDatabaseConnection* self);
FBCMetaData* fbcdcSetMajorityStatus(FBCDatabaseConnection* self, Bool status);
FBCMetaData* fbcdcGetReadOnlyStatus(FBCDatabaseConnection* self);
FBCMetaData* fbcdcSetReadOnlyStatus(FBCDatabaseConnection* self, Bool status);

Bool         fbcdcDatabaseID(FBCDatabaseConnection* self, FSUniqueId* databaseID);
FBTVersion   fbcdcClusterVersion(FBCDatabaseConnection* self);
void         fbcdcVerifyCommitRights(FBCDatabaseConnection* self);

#endif /* FBCDatabaseConnection__H */
