#ifndef FBCExecHandler__H
#define FBCExecHandler__H

/*!
@header FBCExecHandler
 @discussion FBCExecHandler provides a means for connecting to and managing FBExec daemons on a given host.  Using FBCExecHandler, you can retrieve information such as the available databases on a host, their statuses, etc.  You can also create, start, and delete databases.
 */
 

struct FBCExecHandler;
typedef struct FBCExecHandler FBCExecHandler;

/*!
@enum FBCExecClientType
 @discussion The various well known client types which can be registered with FBExec using fbcehHandlerForHostWithClient and friends.
 */
typedef enum FBCExecClientType {
    FBCUnknownType      = 0,
    FBCDatabaseManager  = 1,
    FBCUserManager      = 2,
    FBCServer           = 4,
    FBCExec             = 8,
    FBCClientApp        = 16,
    FBCLasso            = 32,
    FBCReplicator       = 64,
    FBCReplicatorClient = 128
} FBCExecClientType;

/*!
@function fbcehSetMultiThreaded
 @discussion Setting multithreaded to true will cause new instances of FBCExecHandler to be created for each request to fbcehHandlerForHost (and friends).  Also, no permanent connection will be maintained to the FBExec daemon; rather, connections will be reestablished for each request.
 */
void fbcehSetMultiThreaded(Bool flag);

/*!
@function fbcehHandlerForHostWithClientRM
 @discussion Like fbcehHandlerForHostWithClient, except that *errorMsg identifies the error message if an error occurs.  The caller should not free the error message string.
 */
FBCExecHandler* fbcehHandlerForHostWithClientRM(char* hostName, FBCExecClientType clientType, char* clientName, unsigned clientID, char** errorMsg);

/*!
@function fbcehHandlerForHostWithClient
 @discussion Establishes a connection, known as exec handler object, to the FBExec daemon running on the specified host.
 @param hostName Host of the FBExec daemon.
 @param clientType The type of client requesting connection (see FBCExecClientType).
 @param clientName Name of the client tool or application (such as "FrontBase Manager").
 @param clientID Identifier for client.
 @result Returns either a connected FBCExecHandler object, or NULL upon error.  In the event of an error, an error message can be retrieved using fbcehClassErrorMessage().
 */
FBCExecHandler* fbcehHandlerForHostWithClient(char* hostName, FBCExecClientType clientType, char* clientName, unsigned clientID);

/*!
@function fbcehHandlerForHostRM
 @discussion Like fbcehHandlerForHost, except that *errorMsg identifies the error message if an error occurs.  The caller should not free the error message string.
 */
FBCExecHandler* fbcehHandlerForHostRM(char* hostName, FBCExecClientType clientType, char** errorMsg);

/*!
@function fbcehHandlerForHost
 @discussion Convenience for fbcehHandlerForHostWithClient with "any client" and a zero client ID.
 */
FBCExecHandler* fbcehHandlerForHost(char* hostName, FBCExecClientType clientType);

/*!
@function fbcehHandlerForThisHostRM
 @discussion Like fbcehHandlerForThisHost, except that *errorMsg identifies the error message if an error occurs.  The caller should not free the error message string.
 */
FBCExecHandler* fbcehHandlerForThisHostRM(FBCExecClientType clientType, char** errorMsg);

/*!
@function fbcehHandlerForThisHost
 @discussion Convenience for fbcehHandlerForHostWithClient with localhost, "any client" and a zero client ID.
 */
FBCExecHandler* fbcehHandlerForThisHost(FBCExecClientType clientType);

/*!
@function fbcehHandlerForHostWithClientPW
 @discussion Similar to fbcehHandlerForHostWithClientRM, but also takes a clear text password as the FBExec password.
 */
FBCExecHandler* fbcehHandlerForHostWithClientPW(char* hostName, FBCExecClientType clientType, char* clientName, unsigned clientID, char* passwd, char** errorMsg);

/*!
@function fbcehHandlerForHostPW
 @discussion Similar to fbcehHandlerForHostRM, but also takes a clear text password as the FBExec password.
 */
FBCExecHandler* fbcehHandlerForHostPW(char* hostName, FBCExecClientType clientType, char* passwd, char** errorMsg);

/*!
@function fbcehHandlerForThisHostPW
 @discussion Similar to fbcehHandlerForThisHostRM, but also takes a clear text password as the FBExec password.
 */
FBCExecHandler* fbcehHandlerForThisHostPW(FBCExecClientType clientType, char* passwd, char** errorMsg);

/*!
@function fbcehHostname
 @discussion Returns the hostname associated with the given exec handler.  Please note that this is the same hostname that was provided to create the exec handler, not a resolved version of the IP address associated with the socket.  Thus, this could return a "hostname" in dotted quad format, such as "127.0.0.1".
 */
const char* fbcehHostname(FBCExecHandler* self);

/*!
@function fbcehVersion
 @discussion Returns the FBExec version, which can be useful when writing tools which must be compatible with several versions of FrontBase.
 @result The returned string should not be freed by the caller.
 */
const char* fbcehVersion(FBCExecHandler* self);

/*!
 @function fbcehClassErrorMessage
 @discussion Returns an error message describing a connection error.
 @result The returned string should not be freed by the caller.
 */
const char* fbcehClassErrorMessage(void);

/*!
@function fbechErrorMessage
 @discussion Backwards compatible version of fbcehErrorMessage().
 */
char* fbechErrorMessage(FBCExecHandler* self);

/*!
@function fbcehErrorMessage
 @discussion Returns an error message to describe a command failure.
 @result The returned string should not be freed by the caller.
 */
char* fbcehErrorMessage(FBCExecHandler* self);

/*!
@function fbcehDisconnectAll
 @discussion Disconnect all exec handlers.  Most often used when terminating.
 */
void fbcehDisconnectAll(void);

/*!
@function fbcehDisconnect
 @discussion Disconnect the given exec handler.
 */
void fbcehDisconnect(FBCExecHandler* self);

/*!
@function fbcehRelease
 @discussion Releases the resources associated with an FBCExecHandler.
 */
void fbcehRelease(FBCExecHandler* self);

/*!
@function fbcehReleaseThreaded
 @discussion Releases the resources associated with an FBCExecHandler, if called from a multithreaded context (see fbcehSetMultiThreaded)
 */
void fbcehReleaseThreaded(FBCExecHandler* self);

/*!
@function fbcehAvailableDatabases
 @discussion Returns the available databases on the given host.
 @result An FBArray object of the available databases.  When finished, the caller should free this array by calling fbcehReleaseDBNames().
 */
FBArray* fbcehAvailableDatabases(FBCExecHandler* self);

/*!
@function fbcehDBNameAtIndex
 @discussion Returns the name of the database at the given index, or NULL if the index is out of bounds.
 @result The returned string should not be freed by the caller.
 */
char* fbcehDBNameAtIndex(FBArray* availables, int index);

/*!
@function fbcehReleaseDBNames
 @discussion Releases the resources associated with the database array returned from fbcehAvailableDatabases().
 */
void fbcehReleaseDBNames(FBArray* availables);

/*!
@function fbcehStatusForDatabaseNamed
 @discussion Returns the current status of the given database.  See the enum, FBDatabaseStatus.
 */
FBDatabaseStatus fbcehStatusForDatabaseNamed(FBCExecHandler* self, char* dbName);

/*!
@function fbcehPortNumberForDatabaseNamed
 @discussion Returns the port number to use for connecting to the given database.
 */
unsigned fbcehPortNumberForDatabaseNamed(FBCExecHandler* self, char* dbName);

/*!
@function fbcehGetClientID
 @discussion Returns the client ID of the given client.  Returns 0xffffffff upon error.
 */
unsigned fbcehGetClientID(FBCExecHandler* self, char* clientName);

/*!
@function fbcehGetClientIDTyped
 @discussion Returns the client ID of the given client of the given type.  Returns 0xffffffff upon error.
 */
unsigned fbcehGetClientIDTyped(FBCExecHandler* self, char* clientName, FBCExecClientType type);

// Get the latest string of options the given database has been started with.
// It is up to the caller to copy the returned string before the next call to the
// handler. A return value of NULL means that no options string is available.
/*!
@function fbcehGetOptionsForDatabaseNamed
 @discussion Return the latest string of options which the given database was started with.  NULL will be returned if the options string is not available.  Since the string returned is a shared internal buffer, it should not be freed by the caller, and must be copied before a subsequent call to fbcehGetOptionsForDatabaseNamed() if you wish to retain the original information.
 */
const char* fbcehGetOptionsForDatabaseNamed(FBCExecHandler* self, char* dbName);

/*!
@function fbcehStartDatabaseNamed
 @discussion Start the given database.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehStartDatabaseNamed(FBCExecHandler* self, char* dbName);

/*!
@function fbcehStartDatabaseNamedPW
 @discussion Start the given database after authenticating with FBExec using the provided password.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehStartDatabaseNamedPW(FBCExecHandler* self, char* dbName, const char* clearTextPassword);

/*!
@function fbcehStartDatabaseNamedWithOptions
 @discussion Start the given database using the specified startup options.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehStartDatabaseNamedWithOptions(FBCExecHandler* self, char* dbName, const char* options);

/*!
@function fbcehStartDatabaseNamedWithOptionsPW
 @discussion Start the given database with the specified startup options after authenticating with FBExec using the provided password.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehStartDatabaseNamedWithOptionsPW(FBCExecHandler* self, const char* dbName, const char* options, const char* clearTextPassword);

/*!
@function fbcehCreateDatabaseNamed
 @discussion Create a new database.  Note: This method has been deprecated.  Please use fbcehStartDatabaseNamedWithOptions() or similar with the -create option.
 */
Bool fbcehCreateDatabaseNamed(FBCExecHandler* self, char* dbName);

/*!
@function fbcehCreateDatabaseNamedPW
 @discussion Create a new database.  Note: This method has been deprecated.  Please use fbcehStartDatabaseNamedWithOptionsPW() or similar with the -create option.
 */
Bool fbcehCreateDatabaseNamedPW(FBCExecHandler* self, char* dbName, const char* clearTextPassword);

/*!
@function fbcehCreateDatabaseNamedWithOptions
 @discussion Create a new database.  Note: This method has been deprecated.  Please use fbcehStartDatabaseNamedWithOptionsPW() or similar with the -create option.
 */
Bool fbcehCreateDatabaseNamedWithOptions(FBCExecHandler* self, char* dbName, char* options);

/*!
@function fbcehCreateDatabaseNamedWithOptionsPW
 @discussion Create a new database.  Note: This method has been deprecated.  Please use fbcehStartDatabaseNamedWithOptionsPW() or similar with the -create option.
 */
Bool fbcehCreateDatabaseNamedWithOptionsPW(FBCExecHandler* self, const char* dbName, const char* options, const char* clearTextPassword);

/*!
@function fbcehDeleteDatabaseNamed
 @discussion Deletes a database.  Removes the physical database file and associated log file.  Requires that the database be stopped.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehDeleteDatabaseNamed(FBCExecHandler* self, const char* dbName);

/*!
@function fbcehDeleteDatabaseNamedPW
 @discussion Same as fbcehDeleteDatabaseNamed(), but authenticates with FBExec first.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehDeleteDatabaseNamedPW(FBCExecHandler* self, const char* dbName, const char* clearTextPassword);

/*!
@function fbcehKillDatabaseNamed
 @discussion Kills a running database.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehKillDatabaseNamed(FBCExecHandler* self, char* dbName);

/*!
@function fbcehRenameDatabaseNamed
 @discussion Renames the given database file and associated log files.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehRenameDatabaseNamed(FBCExecHandler* self, char* oldName, char* newName);

/*!
@function fbcehRenameDatabaseNamed
 @discussion Same as fbcehRenameDatabaseNamed(), but authenticates with FBExec first.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehRenameDatabaseNamedPW(FBCExecHandler* self, char* oldName, char* newName, const char* clearTextPassword);

// License string issues, a license string is 64 characters and the check string
// is 16 characters. At this level the two strings are joined with a colon, e.g.
// <64 characters>:<16 characters>

// Set a license string for the database server connected via the handler.
/*!
@function fbcehSetLicenseString
 @discussion Sets a license string for the FrontBase installation associated with the connected FBExec daemon.  The license string should be a 64 character string, followed by a colon, followed by a 16 character check string.
 @result Returns True upon success, False upon failure.  If the given license is not valid for some reason, False will be returned.
 */
Bool fbcehSetLicenseString(FBCExecHandler* self, char* licenseWithCheck);

/*!
@function fbcehSetLicenseStringPW
 @discussion Same as fbcehSetLicenseString(), but authenticates with FBExec first.
 */
Bool fbcehSetLicenseStringPW(FBCExecHandler* self, char* licenseWithCheck, const char* clearTextPassword);

/*!
@function fbcehLicenseString
 @discussion Returns the license string for the FrontBase installation associated with the given FBExec daemon.
 @result Returns True upon success, False upon failure.  The returned license string should not be freed by the caller.
 */
const char* fbcehLicenseString(FBCExecHandler* self, char* licenseString);

/*!
@function fbcehLicenseInfoAsPropertyList
 @discussion Returns the license string in the compact Apple property list format.
 @result Returns True upon success, False upon failure.  The returned property list should not be freed by the caller.
 */
const char* fbcehLicenseInfoAsPropertyList(FBCExecHandler* self);

/*!
@function fbcehDigestPassword
 @discussion Digests the given username and password into a 16 character hex string.
 @result Returns the digested string.  The caller should not free the returned string, and if it must be used after the next FBCExecHandler call, it should be copied into a separate buffer.
 */
const char* fbcehDigestPassword(FBCExecHandler* self, const char* userName, const char* password);

/*!
@function fbcehSQLErrorsText
 @discussion Returns the FBSQLErrors.array file, which is a property list based representation of the error message templates.
 @result The caller should not free the returned property list, and if it must be used after the next FBCExecHandler call, it should be copied into a separate buffer.
 */
const char* fbcehSQLErrorsText(FBCExecHandler* self);

/*!
@function fbcehKeywordsText
 @discussion Returns the Keywords.txt file, which is a list of all keywords recognised by FrontBase.
 @result The caller should not free the returned string, and if it must be used after the next FBCExecHandler call, it should be copied into a separate buffer.
 */
const char* fbcehKeywordsText(FBCExecHandler* self);

/*!
@function fbcehGetAutoStartInfo
 @discussion Returns an FBCAutoStartInfo object representing the list of databases (and their start options) which will be automatically started when the FBExec daemon is started with the -autostart option.
 @result An FBCAutoStartInfo object.  The caller is responsible for releasing this object using fbcaiRelease().
 */
FBCAutoStartInfo* fbcehGetAutoStartInfo(FBCExecHandler* self);

/*!
@function fbcehStartFBAppPW
 @discussion Internal administration API.  Not intended for use by the public.
 */
const char* fbcehStartFBAppPW(FBCExecHandler* self, char* command, char* options, char* argument, char* fbexecPasswd);

/*!
@function fbcehSetAutoStartInfo
 @discussion Specifies the FBCAutoStartInfo for the associated FBExec daemon.  See fbcehGetAutoStartInfo() and FBCAutoStartInfo.h for more details.
 @result Returns True upon success, False upon failure.
 */
Bool fbcehSetAutoStartInfo(FBCExecHandler* self, FBCAutoStartInfo* startInfo, char* fbexecPasswd);

/*!
@function fbcehGetDatabaseLogTail
 @discussion Get the newst entry in the log file for the given database.
 @result The returned log string should not be freed by the caller.
 */
const char* fbcehGetDatabaseLogTail(FBCExecHandler* self, char* shortDatabaseName);

/*!
@function fbcehGetTransactionVersion
 @discussion Get the transaction version for the named database
 @result The transaction version is returned, -1 when an error occurs
 */
FBTVersion fbcehGetTransactionVersion ( FBCExecHandler* self, char* name );

/*!
@function fbcehGetClusterDescr
 @discussion Get the ClusterDescr for the named database
 @result The ClusterDescr for the database, NULL when an error occurs
 */
char* fbcehGetClusterDescr ( FBCExecHandler* self, char* name );

/*!
@function fbcehGetDatabaseId
 @discussion Get the FSUniqueId for the named database
 @result If the FSUniqueId for the named database may be obtained, it is assigned to 'databaseID'
 */
void fbcehGetDatabaseId ( FBCExecHandler* self, char* name, FSUniqueId* databaseID );

/*!
@function fbcehSetMemberNames
 @discussion Set the MemberNames part of the ClusterDescr for the named database
 @result "OK" for success, error message otherwise
 */
char* fbcehSetMemberNames ( FBCExecHandler* self, char* name, char* memberNames, char* execPW );

/*!
@function fbcehSetMajorityStatus
 @discussion Set the Majority part of the ClusterDescr for the named database
 @result "OK" for success, error message otherwise
 */
char* fbcehSetMajorityStatus ( FBCExecHandler* self, char* name, Bool status, char* execPW );

/*!
@function fbcehRemoveClusterOption
 @discussion Remove "-rcluster" from the options file of the named database
 @result "OK" for success, error message otherwise
 */
char* fbcehRemoveClusterOption ( FBCExecHandler* self, char* shortDatabaseName, char* execPW );

/*!
@function fbcehSendCommandToExec
 @discussion Internal API for FBCExecHandler use only.  Not intended for use by the public.
 */
char* fbcehSendCommandToExec(FBCExecHandler* self, char* cmd);

#endif /* FBCExecHandler__H */
