#ifndef FBCMetaData__H
#define FBCMetaData__H


/*!
@header FBCMetaData
 @discussion FBCMetaData is a general meta data object containing auxiliary information about a database operation.  In almost all cases, FBCMetaData will indicate success or failure of the operation through fbcmdErrorsFound().  In some cases, such as after executing a SQL statement using fbcdcExecuteDirectSQL(), the meta data object will identify the type of statement, the row and column count affected, the execution time, etc.
*/

/*!
@function fbcmdMetaDataFromResultString
 @discussion Internal API for constructing FBCMetaData instances.  Not intended for public use.
 */
FBCMetaData* fbcmdMetaDataFromResultString(FBCDatabaseConnection* fbdc, char* resultString);

/*!
@function fbcmdCatalogName
 @discussion Return the SQL 92 catalog name associated with the previous operation.
 @result The returned string should not be freed by the caller.
 */
char* fbcmdCatalogName(const FBCMetaData* self);

/*!
@function fbcmdMessage
 @discussion A special message area which is sometimes used by the server to return results which don't fit within the normal SQL result set scheme.  For instance, after executing an extract statement, the meta data message will contain a property list describing the result.
 @result The returned string should not be freed by the caller.
 */
char* fbcmdMessage(const FBCMetaData* self);

/*!
@function fbcmdExecutionTime
 @discussion The server execution time for the previous operation.  Please note that the server can choose to evaluate some statements lazily, so deriving the total time required for a given statement can involve the timing the fetch stage.
 @result The returned string should not be freed by the caller.
 */
char* fbcmdExecutionTime(const FBCMetaData* self);

/*!
@function fbcmdFetchHandle
 @discussion The fetch handle associated with the previous query.  This should be passed to fbcdcFetch().
 @result The returned string should not be freed by the caller.
 */
char* fbcmdFetchHandle(const FBCMetaData* self);

/*!
@function fbcmdTimeZone
 @discussion The time zone associated with the previous operation.
 @result The returned string should not be freed by the caller.
 */
char* fbcmdTimeZone(const FBCMetaData* self);

char* fbcmdStatementHandle(const FBCMetaData* self);

/*!
@function fbcmdStatementType
 @discussion The type of SQL statement for the previous block of executed SQL, such as SELECT, UPDATE, DELETE, etc.
 @result The returned string should not be freed by the caller.
 */
char* fbcmdStatementType(const FBCMetaData* self);

/*!
@function fbcmdErrorClass
 @discussion The class of error for the previous operation.  Assumes that the previous operation failed.
 @result The returned string should not be freed by the caller.
 */
char* fbcmdErrorClass(const FBCMetaData* self);

FBCBlobHandle* fbcmdBlobHandle(const FBCMetaData* self);

/*!
@function fbcmdErrorsFound
 @discussion Returns True if errors were found during the execution of an SQL statement.
 */
Bool fbcmdErrorsFound(const FBCMetaData* self);

/*!
@function fbcmdWarningsFound
 @discussion Returns True if warnings were found during the execution of an SQL statement.
 */
Bool fbcmdWarningsFound(const FBCMetaData* self);

/*!
@function fbcmdRowCount
 @discussion Returns the number of rows fetched by the previous SQL statement.
 */
int fbcmdRowCount(const FBCMetaData* self);

/*!
@function fbcmdRowIndex
 @discussion Returns the row index of the row just inserted.  This is only valid if the statement executed was an INSERT statement inserting a single row.
 */
int fbcmdRowIndex(const FBCMetaData* self);

/*!
@function fbcmdColumnCount
 @discussion Returns the number of columns fetched by the executed SQL statement.
 */
unsigned fbcmdColumnCount(const FBCMetaData* self);

/*!
@function fbcmdErrorCount
 @discussion Returns the number of errors found during execution of an SQL statement.
 */
unsigned fbcmdErrorCount(const FBCMetaData* self);

/*!
@function fbcmdPlanInfo
 @discussion Returns the query plan information string (NULL if none is available).
 @result The returned string should not be freed by the caller.
 */
const char* fbcmdPlanInfo(const FBCMetaData* self);

unsigned fbcmdBatchSize(const FBCMetaData* self);

unsigned fbcmdUniqueKey(const FBCMetaData* self);

FBTVersion fbcmdTransactionVersion(const FBCMetaData* self);

const char* fbcmdVersion ( const FBCMetaData* self );

const char* fbcmdACID ( const FBCMetaData* self );


/*!
@function fbcmdDatabaseConnection
 @discussion Returns the database connection associated with the meta data.
 @result The returned database connection is a reference, not a new instance, so it should not be freed by the caller.
 */
FBCDatabaseConnection* fbcmdDatabaseConnection(const FBCMetaData* self);

/*!
@function fbcmdColumnMetaDataAtIndex
 @discussion Returns the column meta data for the column at the specified index.
 @result The returned FBCColumnMetaData should not be released by the caller.
 */
const FBCColumnMetaData* fbcmdColumnMetaDataAtIndex(const FBCMetaData* self, unsigned index);

/*!
@function fbcmdDatatypeMetaDataAtIndex
 @discussion Returns the data type meta data for the column at the specified index.
 @result The returned FBCDatatypeMetaData should not be released by the caller.
 */
const FBCDatatypeMetaData* fbcmdDatatypeMetaDataAtIndex(const FBCMetaData* self, unsigned index);

/*!
@function fbcmdTakeFetchMetaDataFrom
 @discussion Internal API.  Not intended for public use.
 */
void fbcmdTakeFetchMetaDataFrom( FBCMetaData* self, FBCMetaData* fetchMetaData);

/*!
@function fbcmdDescription
 @discussion Debugging description of the meta data object.
 @result The returned string should be freed by the caller.
 */
char* fbcmdDescription( const FBCMetaData* self);

/*!
@function fbcmdHasMetaDataArray
 @discussion Returns True if there are multiple meta data objects associated with the previous operation (indicating multiple result sets).  See fbcmdMetaDataAtIndex() for more information.
 */
Bool fbcmdHasMetaDataArray(const FBCMetaData* self);

/*!
@function fbcmdMetaDataArrayCount
 @discussion Returns the number of meta data objects associated with the previous operation.
 */
unsigned fbcmdMetaDataArrayCount(const FBCMetaData* self);

/*!
@function fbcmdMetaDataAtIndex
 @discussion Returns the meta data object at the given index for the previous operation.  Some operations can return multiple result sets, each of which are accessible by a single meta data.  For instance, executing the SQL, "select * from t0; select * from t1;", will return two result sets.  The result sets need not be similar.
 */
FBCMetaData* fbcmdMetaDataAtIndex(const FBCMetaData* self, unsigned indx);

/*!
@function fbcmdAllRowsFetched
 @discussion Returns a boolean value telling whether all rows of the associated result set has been fetched. The flag is set based on the meta data associated with calls to fbcdcExecuteDirectSQLWithFetch and fbcdcFetch. This is a good way to avoid another call to fbcdcFetch (and fbcdcCancelFetch) just to detect that there are no more rows.
 */
Bool fbcmdAllRowsFetched(const FBCMetaData* self);

/*!
@function fbcmdRelease
 @discussion Releases resources associated with the FBCMetaData.
 */
void fbcmdRelease(FBCMetaData* self);

/*!
@function fbcmdMemberNames
 @discussion Returns the cluster ActiveMember list as a result of a call to fbcdcMemberNames.
 */
const char* fbcmdMemberNames ( const FBCMetaData* self );

/*!
@function fbcmdReadOnlyStatus
 @discussion Returns the ReadOnly status of a clustermember as a result of a call to fbcdcReadOnlyStatus.
 */
Bool fbcmdReadOnlyStatus ( const FBCMetaData* self );

/*!
@function fbcmdMajorityStatus
 @discussion Returns the Majority status of a clustermember as a result of a call to fbcdcMajorityStatus.
 */
Bool fbcmdMajorityStatus ( const FBCMetaData* self );

/*!
@function fbcmdDatabaseID
 @discussion Sets databaseID to the system identification obtained by a call to fbcdcDatabaseID. Returns True on success
 */
Bool fbcmdDatabaseID ( const FBCMetaData* self, FSUniqueId* databaseID );

/*!
@function fbcmdClusterVersion
 @discussion Returns the transaction version of a clustermember as a result of a call to fbcdcClusterVersion.
 */
FBTVersion fbcmdClusterVersion ( const FBCMetaData* self );

/*!
@function fbcmdFetch
 @discussion Fetch the next batch of data for the meta data, updated the row count, the batch size, and all rows fetched
 if no data is available, Null is returned.
 */
void* fbcmdFetch ( FBCMetaData* self, unsigned rowCount );

/*!
@function fbcmdCancelFetch
 @discussion Cancel a possible fetch on the metadata
 */
FBCMetaData* fbcmdCancelFetch ( FBCMetaData* self );
void**       fbcmdFetchRow    ( FBCMetaData* self );
void**       fbcmdRowAtIndex  ( FBCMetaData* self, unsigned int index );
void         fbcmdFetchRows   ( FBCMetaData* self );

#endif /* FBCMetaData__H */
