#ifndef FBCRowHandler__H
#define FBCRowHandler__H

/*!
@header FBCRowHandler
 @abstract FBCRowHandler is used to convert compact fetch data into a form that is readily usable by the C language.

 @discussion After using FBCDatabaseConnection's fbcdcExecuteDirectSQL() and fbcdcFetch() calls to retrieve fetch data, you must use FBCRowHandler to extract that data and convert into a useful form.  Row handlers can convert fetch data into different forms depending on how they are initialized.  For instance, for a TIMESTAMP column, a row handler can return either a formatted string representing the timestamp, or an unformatted value representing the displacement in seconds from a fixed time.
*/

/*!
@function fbcrhInitWith
 @discussion Creates a new FBCRowHandler using the given fetch data (see fbcdcFetch()) and a meta data object describing the fetch result (see fbcdcExecuteDirectSQL()).  Returns NULL if the row handler cannot be created.
*/
FBCRowHandler* fbcrhInitWith(void* rawData, const FBCMetaData* meta);

/*!
@function fbcrhInitWithSkipFormatting
 @discussion Similar to fbcrhInitWith(), but can optionally skip formatting for SQL types such as DATE, TIMESTAMP, etc.  By default, fbcrhInitWith() formats these types.
 */
FBCRowHandler* fbcrhInitWithSkipFormatting(void* rawData, const FBCMetaData* meta, Bool skipFormatting);

/*!
@function fbcrhInitWithOptions
 @discussion Similar to fbcrhInitWith().  Allows the skipping of formatting for SQL types such as DATE, TIMESTAMP, etc.  Allows the return numeric values, such as INTEGER and DECIMAL, as C strings.  Allows the caller to configure the timezone used to build TIME WITH TIME ZONE and TIMESTAMP WITH ZONE values.
 */
FBCRowHandler* fbcrhInitWithOptions(void* rawData, const FBCMetaData* meta, Bool skipFormatting, Bool numberAsChar, Bool fetchTimeZone);

/*!
@function fbcrhAddBatch
 @discussion Used to add a fetch batch to the row handler.
 @result Returns False if there are no rows and new batch is not added.  Otherwise, returns True.
*/
Bool fbcrhAddBatch(FBCRowHandler* self, void* rawData);

/*!
@function fbcrhRowCount
 @discussion Returns the count of all rows associated with the row handler.
*/
unsigned fbcrhRowCount(FBCRowHandler* self);

/*!
@function fbcrhColumnCount
 @discussion Returns the column count of the rows associated with the row handler.
 */
unsigned fbcrhColumnCount(FBCRowHandler* self);

/*!
@function fbcrhFirstRow
 @discussion Returns the first row within the row handler.  See description of fbcrhRowAtIndex() for more details.
 */
void** fbcrhFirstRow(FBCRowHandler* self);

/*!
@function fbcrhNextRow
 @discussion Returns the next row within the row handler.  See description of fbcrhRowAtIndex() for more details.
 */
void** fbcrhNextRow(FBCRowHandler* self);

/*!
@function fbcrhRowAtIndex
 @discussion Returns the row at the given index within the row handler, or NULL if there are no more rows or index is out of bounds.

 Each row is returned as an array of void pointers, with the number of elements equal to the fbcrhColumnCount().  Each void pointer identifies a column value, with NULL identifying a NULL value.  Column values should be cast as follows, depending on their associated SQL type:

 FB_Boolean: *(unsigned char*) row[<col>]
 FB_PrimaryKey: *(int*) row[<col>]	This is an internal datatype.
 FB_Integer: *(int*) row[<col>]
 FB_TinyInteger: *(unsigned char*) row[<col>]
 FB_SmallInteger: *(short*) row[<col>]
 FB_LongInteger: *(FBLongInteger*) row[<col>]
 FB_Numeric: *(double*) row[<col>]
 FB_Decimal: *(double*) row[<col>]
 FB_Real: *(double*) row[<col>]
 FB_Float: *(double*) row[<col>]
 FB_Double: *(double*) row[<col>]

 FB_Date:
 When formatting is enabled: (char*) row[<col>] (formatted as YYYY-MM-DD)
 When formatting is disabled: *(double*)row[<col>] (seconds since 2001-01-01 00:00:00)
 
 FB_Time:
 When formatting is enabled: (char*) row[<col>] (formatted as HH:MM:SS)
 When formatting is disabled: (FBCUnformattedTime*)row[<col>] (seconds since 00:00:00)
 
 FB_TimeTZ:
 When formatting is enabled: (char*) row[<col>] (formatted as HH:MM:SS<sign>MM:SS)
 When formatting is disabled: (FBCUnformattedTime*)row[<col>] (seconds since 00:00:00)

 FB_Timestamp:
 When formatting is enabled: (char*) row[<col>]	(formatted  as YYYY-MM-DD HH:MM:SS)
 When formatting is disabled: (FBCUnformattedTime*) row[<col>] (seconds since 2001-01-01 00:00:00)
 
 FB_TimestampTZ:
 When formatting is enabled: (char*) row[<col>]	(formatted  as YYYY-MM-DD HH:MM:SS<sign>MM:SS)
 When formatting is disabled: (FBCUnformattedTime*) row[<col>] (seconds since 2001-01-01 00:00:00)
 
 FB_YearMonth: *(int*) row[<col>]
 FB_DayTime: *(double*) row[<col>]
 FB_Character: (char*) row[<col>]
 FB_VCharacter: (char*) row[<col>]
 FB_Bit: (FBCBitValue*) row[<col>]
 FB_VBit: (FBCBitValue*) row[<col>]
 FB_CLOB: (FBCBlobIndirect*) row[<col>]
 FB_BLOB: (FBCBlobIndirect*) row[<col>]

 FB_CircaDate: (FBCCircaDate)row[col]

 @result Return row data is kept within a buffer internal to the FBCRowHandler and should not be freed.  Row data resources will be freed when the row handler itself is released using  fbcrhRelease().
 */
void** fbcrhRowAtIndex(FBCRowHandler* self, unsigned row);

/*!
@function fbcrhConvertToOutputCharSet
@discussion Converts the given string to the given output character set.
*/
unsigned char* fbcrhConvertToOutputCharSet(int outputCharacterSet, unsigned char* s);

/*!
@function fbcrhRelease
 @discussion Frees the resources associated with an FBCRowHandler.
*/
void fbcrhRelease(FBCRowHandler* self);

/*!
@function fbdecConvertFromString
 @discussion Internal API to FBCAccess for converting values to FBDecimals.
*/
FBDecimal fbdecConvertFromString(FBChar* s, unsigned* precision);

/*!
@struct FBCBitValue
 @field size Size of the bit string measured in bytes.
 @field bytes Pointer to an array of bytes.
*/
typedef struct FBCBitValue {
	unsigned size;
	unsigned char* bytes;
} FBCBitValue;

/*!
@struct FBCUnformattedTime
 @field seconds For TIMESTAMP values, this is the seconds since 2001-01-01 00:00:00.  For TIME values, this is the seconds since 00:00:00.
 @field tzSign Sign of the timezone displacement
 @field tzValue Unsigned timezone displacement
 */
typedef struct FBCUnformattedTime {
	double seconds;
	unsigned char tzSign;
	unsigned int  tzValue;
} FBCUnformattedTime;

/*!
@struct FBCBlobDirect
 @discussion Direct blob references have been deprecated, so this structure should not be used.
*/
typedef struct FBCBlobDirect {
	unsigned char kind;
	unsigned int  blobSize;
	void* blobData;
} FBCBlobDirect;

/*!
@struct FBCBlobIndirect
 @field kind No longer necessary since direct blob references have been deprecated.
 @field handleAsString String version of the indirect blob handle.  Formatted like: '<24 hex digits>'\0.  See FBCBlobHandle.h for more information.
 */
typedef struct FBCBlobIndirect {
	unsigned char kind;
	char handleAsString[28];
} FBCBlobIndirect;


/*!
@struct FBCCircaDate
 @field Values since 2001-01-01
 */
typedef struct FBCCircaDate {
	double lower;
	double upper;
} FBCCircaDate;

#endif /* FBCRowHandler__H */
