#ifndef FBCXML__H
#define FBCXML__H

/*
	FBCXML.h
	Brad Hutchings
	2001 FrontBase, Inc.

	This unit implements XML support in FBCAccess:
	
		1. Generates XML Schema from meta data.
		2. Generates XML from meta data and row data.
		
	Phase 1: Implemented
	--------------------
		* Given a result set (meta data and row handler), dump an XML schema and data.

	Standards Reference
	-------------------
	[XML0]	XML Schema Part 0: Primer				http://www.w3.org/TR/xmlschema-0/
	[XML1]	XML Schema Part 1: Structures			http://www.w3.org/TR/xmlschema-1/
	[XML2]	XML Schema Part 2: Datatypes			http://www.w3.org/TR/xmlschema-2/
*/

//================================================================================
//	Includes
//================================================================================

	#ifndef _STDIO_H
	#include <stdio.h>
	#endif

//================================================================================
//	fbcxml methods
//================================================================================

//	FBCXML error codes

typedef enum FBCXML_Error
{
	FBCXML_NoError			= 0,
	FBCXML_InvalidResult	= 1,			//	a FBCXMLResult was not valid
	FBCXML_NULLParam		= 2,			//	a NULL FBCXMLResult param passed unexpectedly
	FBCXML_CantAllocate		= 3,			//	couldn't allocate memory
	FBCXML_MetaDataError	= 4,
	FBCXML_BlobError		= 5,
	FBCXML_ConnectionError	= 6,			//	NULL connection parameter
	FBCXML_UnknownError		= 127			//	your guess is as good as mine
} FBCXML_Error;

//	FBCXMLResult, based on FSVString
//	FSVString IS NOT exposed in the public API, so declare it here
struct FSVString;
typedef struct FSVString* FBCXMLResult;

//	fbcxmlNewResult
//	
//	This function creates a FBCXMLResult structure. 
//		xml_result		- Pointer to your FBCXMLResult* pointer to create.
//	Function returns an error code, if any, or kFBCXML_NoErr

FBCXML_Error	fbcxmlNewResult(
					FBCXMLResult*	xml_result
				);

//	fbcxmlMetaDataToXMLSchema
//	
//	This function takes an FBCMetaData and generates an XML schema to either or both of
//	an FBCXMLResult (internal structure) and a file.
//		meta_data		- the meta data input
//		xml_result		- generated result, can be NULL if you prefer to use file
//							!! Be sure to fbcxmlFreeResult your results after each use !!
//		xml_out			- generated result output here, can be NULL
//		result_set_tag	- tag for main result set, pass NULL to use "results"
//		result_tag		- tag for each result, pass NULL to use "result"
//	Function returns an error code, if any, or kFBCXML_NoErr

FBCXML_Error	fbcxmlMetaDataToXMLSchema(
					const FBCMetaData*	meta_data,
					FBCXMLResult		xml_result,
					FILE*				xml_out,
					const char*			result_set_tag,
					const char*			result_tag
				);

//	fbcxmlResultsToXML
//	
//	This function takes an FBCMetaData and and an FBCRowHandler and generates
//	XML for the result set to either or both of an FBCXMLResult (internal structure)
//	and a file.
//		meta_data		- the meta data input
//		row_handler		- a row_handler generated with skipFormatting=NO and numberAsChar=NO
//							(fbcrhInitWith() is prefered call)
//		xml_result		- generated result, can be NULL if you prefer to use file
//							!! Be sure to fbcxmlFreeResult your results after each use !!
//		xml_out			- generated result output here, can be NULL
//		result_set_tag	- tag for main result set, pass NULL to use "results"
//		result_tag		- tag for each result, pass NULL to use "result"
//	Function returns an error code, if any, or kFBCXML_NoErr

FBCXML_Error	fbcxmlResultsToXML(
					const FBCMetaData*		meta_data,
					const FBCRowHandler*	row_handler,
					FBCXMLResult			xml_result,
					FILE*					xml_out,
					const char*				result_set_tag,
					const char*				result_tag
				);

//	fbcxmlFreeResult
//	
//	This function frees an FBCXMLResult structure. Be sure to call this after
//	your done with the FBCXMLResult after each call to fbcxmlMetaDataToXMLSchema and
//	fbcxmlResultsToXML.
//		result		- FBCXMLResult to free
//	Function returns an error code, if any, or kFBCXML_NoErr

FBCXML_Error	fbcxmlFreeResult(
					FBCXMLResult	xml_result
				);

//	fbcxmlResultText
//	
//	This function returns a const pointer to the null terminated text which makes
//	up the XML result.
//		result		- FBCXMLResult to use
//	Function returns a string. Do not free!

const char*		fbcxmlResultText(
					const FBCXMLResult	xml_result
				);

#endif /* FBCXML__H */
